@@ -1,4 +1,4 @@
-/* Copyright 1995-1996,1998,2000-2001,2004,2006,2008-2011,2014,2018
+/* Copyright 1995-1996,1998,2000-2001,2004,2006,2008-2011,2014,2018-2019
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -460,7 +460,7 @@ SCM_DEFINE (scm_char_upcase, "char-upcase", 1, 0, 0,
 #define FUNC_NAME s_scm_char_upcase
 {
   SCM_VALIDATE_CHAR (1, chr);
-  return SCM_MAKE_CHAR (scm_c_upcase (SCM_CHAR (chr)));
+  return scm_i_make_char (scm_c_upcase (SCM_CHAR (chr)));
 }
 #undef FUNC_NAME
 
@@ -471,7 +471,7 @@ SCM_DEFINE (scm_char_downcase, "char-downcase", 1, 0, 0,
 #define FUNC_NAME s_scm_char_downcase
 {
   SCM_VALIDATE_CHAR (1, chr);
-  return SCM_MAKE_CHAR (scm_c_downcase (SCM_CHAR(chr)));
+  return scm_i_make_char (scm_c_downcase (SCM_CHAR(chr)));
 }
 #undef FUNC_NAME
 
@@ -481,7 +481,7 @@ SCM_DEFINE (scm_char_titlecase, "char-titlecase", 1, 0, 0,
 #define FUNC_NAME s_scm_char_titlecase
 {
   SCM_VALIDATE_CHAR (1, chr);
-  return SCM_MAKE_CHAR (scm_c_titlecase (SCM_CHAR(chr)));
+  return scm_i_make_char (scm_c_titlecase (SCM_CHAR(chr)));
 }
 #undef FUNC_NAME
 
