@@ -1,7 +1,7 @@
 #ifndef SCM_KEYWORDS_H
 #define SCM_KEYWORDS_H
 
-/* Copyright 1995-1996,1999-2001,2006,2008,2015,2018
+/* Copyright 1995-1996,1999-2001,2006,2008,2015,2018-2019
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -42,11 +42,11 @@ SCM_API SCM scm_from_utf8_keyword (const char *name);
 
 #define SCM_KEYWORD(c_name, scheme_name) \
 SCM_SNARF_HERE(static SCM c_name) \
-SCM_SNARF_INIT(c_name = scm_from_locale_keyword (scheme_name))
+SCM_SNARF_INIT(c_name = scm_from_utf8_keyword (scheme_name))
 
 #define SCM_GLOBAL_KEYWORD(c_name, scheme_name) \
 SCM_SNARF_HERE(SCM c_name) \
-SCM_SNARF_INIT(c_name = scm_from_locale_keyword (scheme_name))
+SCM_SNARF_INIT(c_name = scm_from_utf8_keyword (scheme_name))
 
 enum scm_keyword_arguments_flags
 {
