@@ -32,6 +32,7 @@
 #include <unistr.h>
 #include <uniconv.h>
 #include <c-strcase.h>
+#include <intprops.h>
 
 #include "chars.h"
 #include "deprecation.h"
@@ -129,6 +130,12 @@ make_stringbuf (size_t len)
     lenhist[1000]++;
 #endif
 
+  /* Make sure that the total allocation size will not overflow size_t,
+     with ~30 extra bytes to spare to avoid an overflow within the
+     allocator.  */
+  if (INT_ADD_OVERFLOW (len, STRINGBUF_HEADER_BYTES + 32))
+    scm_num_overflow ("make_stringbuf");
+
   buf = SCM_PACK_POINTER (scm_gc_malloc_pointerless (STRINGBUF_HEADER_BYTES + len + 1,
 					    "string"));
 
@@ -155,9 +162,16 @@ make_wide_stringbuf (size_t len)
     lenhist[1000]++;
 #endif
 
+  /* Make sure that the total allocation size will not overflow size_t,
+     with ~30 extra bytes to spare to avoid an overflow within the
+     allocator.  */
+  if (len > (((size_t) -(STRINGBUF_HEADER_BYTES + 32 + sizeof (scm_t_wchar)))
+             / sizeof (scm_t_wchar)))
+    scm_num_overflow ("make_wide_stringbuf");
+
   raw_len = (len + 1) * sizeof (scm_t_wchar);
   buf = SCM_PACK_POINTER (scm_gc_malloc_pointerless (STRINGBUF_HEADER_BYTES + raw_len,
-					    "string"));
+                                                     "string"));
 
   SCM_SET_CELL_TYPE (buf, STRINGBUF_TAG | STRINGBUF_F_WIDE);
   SCM_SET_CELL_WORD_1 (buf, (scm_t_bits) len);
@@ -1218,9 +1232,9 @@ SCM_DEFINE (scm_string_ref, "string-ref", 2, 0, 0,
     scm_out_of_range (NULL, k);
 
   if (scm_i_is_narrow_string (str))
-    return scm_c_make_char (scm_i_string_chars (str)[idx]);
+    return scm_i_make_char (scm_i_string_chars (str)[idx]);
   else
-    return scm_c_make_char (scm_i_string_wide_chars (str)[idx]);
+    return scm_i_make_char (scm_i_string_wide_chars (str)[idx]);
 }
 #undef FUNC_NAME
 
@@ -1230,9 +1244,9 @@ scm_c_string_ref (SCM str, size_t p)
   if (p >= scm_i_string_length (str))
     scm_out_of_range (NULL, scm_from_size_t (p));
   if (scm_i_is_narrow_string (str))
-    return scm_c_make_char (scm_i_string_chars (str)[p]);
+    return scm_i_make_char (scm_i_string_chars (str)[p]);
   else
-    return scm_c_make_char (scm_i_string_wide_chars (str)[p]);
+    return scm_i_make_char (scm_i_string_wide_chars (str)[p]);
 
 }
 
@@ -1393,8 +1407,8 @@ SCM_DEFINE (scm_string_append, "string-append", 0, 0, 1,
       s = SCM_CAR (l);
       SCM_VALIDATE_STRING (SCM_ARGn, s);
       len = scm_i_string_length (s);
-      if (((size_t) -1) - total < len)
-        scm_num_overflow (s_scm_string_append);
+      if (INT_ADD_OVERFLOW (total, len))
+        scm_num_overflow (FUNC_NAME);
       total += len;
       if (!scm_i_is_narrow_string (s))
         wide = 1;
@@ -2181,11 +2195,13 @@ scm_to_stringn (SCM str, size_t *lenp, const char *encoding,
   char *buf;
   size_t ilen, len, i;
   int ret;
-  const char *enc;
 
   if (!scm_is_string (str))
     scm_wrong_type_arg_msg (NULL, 0, str, "string");
 
+  if (encoding == NULL)
+    encoding = "ISO-8859-1";
+
   if (c_strcasecmp (encoding, "UTF-8") == 0)
     /* This is the most common case--e.g., when calling libc bindings
        while using a UTF-8 locale.  */
@@ -2233,13 +2249,10 @@ scm_to_stringn (SCM str, size_t *lenp, const char *encoding,
 
   buf = NULL;
   len = 0;
-  enc = encoding;
-  if (enc == NULL)
-    enc = "ISO-8859-1";
   if (scm_i_is_narrow_string (str))
     {
       ret = mem_iconveh (scm_i_string_chars (str), ilen,
-                         "ISO-8859-1", enc,
+                         "ISO-8859-1", encoding,
                          (enum iconv_ilseq_handler) handler, NULL,
                          &buf, &len);
 
@@ -2252,7 +2265,7 @@ scm_to_stringn (SCM str, size_t *lenp, const char *encoding,
     }
   else
     {
-      buf = u32_conv_to_encoding (enc,
+      buf = u32_conv_to_encoding (encoding,
                                   (enum iconv_ilseq_handler) handler,
                                   (uint32_t *) scm_i_string_wide_chars (str),
                                   ilen,
