@@ -1,22 +1,21 @@
-/* Copyright (C) 1995, 1996, 1997, 1998, 2000, 2001, 2002, 2003, 2004,
- *   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013,
- *   2014 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1998,2000-2014,2018-2019
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -24,50 +23,52 @@
 # include <config.h>
 #endif
 
-#include "libguile/bdw-gc.h"
-#include <gc/gc_mark.h>
-#include "libguile/_scm.h"
-#include "libguile/deprecation.h"
-
-#include <stdlib.h>
-#include <unistd.h>
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <full-read.h>
+#include <nproc.h>
 #include <stdio.h>
-
-#ifdef HAVE_STRING_H
+#include <stdlib.h>
 #include <string.h>   /* for memset used by FD_ZERO on Solaris 10 */
-#endif
-
-#if HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif
+#include <unistd.h>
 
 #if HAVE_PTHREAD_NP_H
 # include <pthread_np.h>
 #endif
 
-#include <sys/select.h>
+#include "async.h"
+#include "bdw-gc.h"
+#include "boolean.h"
+#include "continuations.h"
+#include "deprecation.h"
+#include "dynwind.h"
+#include "eval.h"
+#include "extensions.h"
+#include "fluids.h"
+#include "gc-inline.h"
+#include "gc.h"
+#include "gsubr.h"
+#include "hashtab.h"
+#include "init.h"
+#include "iselect.h"
+#include "jit.h"
+#include "list.h"
+#include "modules.h"
+#include "numbers.h"
+#include "pairs.h"
+#include "ports.h"
+#include "scmsigs.h"
+#include "strings.h"
+#include "symbols.h"
+#include "variable.h"
+#include "version.h"
+#include "vm.h"
 
-#include <assert.h>
-#include <fcntl.h>
-#include <nproc.h>
+#include "threads.h"
 
-#include "libguile/validate.h"
-#include "libguile/eval.h"
-#include "libguile/async.h"
-#include "libguile/ports.h"
-#include "libguile/threads.h"
-#include "libguile/dynwind.h"
-#include "libguile/iselect.h"
-#include "libguile/fluids.h"
-#include "libguile/continuations.h"
-#include "libguile/gc.h"
-#include "libguile/gc-inline.h"
-#include "libguile/init.h"
-#include "libguile/scmsigs.h"
-#include "libguile/strings.h"
-#include "libguile/vm.h"
-
-#include <full-read.h>
+#include <gc/gc_mark.h>
 
 
 
@@ -81,7 +82,7 @@ thread_mark (GC_word *addr, struct GC_ms_entry *mark_stack_ptr,
              struct GC_ms_entry *mark_stack_limit, GC_word env)
 {
   int word;
-  const struct scm_i_thread *t = (struct scm_i_thread *) addr;
+  struct scm_thread *t = (struct scm_thread *) addr;
 
   if (SCM_UNPACK (t->handle) == 0)
     /* T must be on the free-list; ignore.  (See warning in
@@ -98,26 +99,21 @@ thread_mark (GC_word *addr, struct GC_ms_entry *mark_stack_ptr,
      but GC doesn't know to trace them (as they are pointerless), so we
      need to do that here.  See the comments at the top of libgc's
      gc_inline.h.  */
-  if (t->pointerless_freelists)
+  for (size_t n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
     {
-      size_t n;
-      for (n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
+      void *chain = t->pointerless_freelists[n];
+      if (chain)
         {
-          void *chain = t->pointerless_freelists[n];
-          if (chain)
-            {
-              /* The first link is already marked by the freelist vector,
-                 so we just have to mark the tail.  */
-              while ((chain = *(void **)chain))
-                mark_stack_ptr = GC_mark_and_push (chain, mark_stack_ptr,
-                                                   mark_stack_limit, NULL);
-            }
+          /* The first link is already marked by the thread itsel, so we
+             just have to mark the tail.  */
+          while ((chain = *(void **)chain))
+            mark_stack_ptr = GC_mark_and_push (chain, mark_stack_ptr,
+                                               mark_stack_limit, NULL);
         }
     }
 
-  if (t->vp)
-    mark_stack_ptr = scm_i_vm_mark_stack (t->vp, mark_stack_ptr,
-                                          mark_stack_limit);
+  mark_stack_ptr = scm_i_vm_mark_stack (&t->vm, mark_stack_ptr,
+                                        mark_stack_limit);
 
   return mark_stack_ptr;
 }
@@ -251,11 +247,11 @@ thread_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
     unsigned short us;
     unsigned int   ui;
     unsigned long  ul;
-    scm_t_uintmax  um;
+    uintmax_t  um;
   } u;
-  scm_i_thread *t = SCM_I_THREAD_DATA (exp);
+  scm_thread *t = SCM_I_THREAD_DATA (exp);
   scm_i_pthread_t p = t->pthread;
-  scm_t_uintmax id;
+  uintmax_t id;
   u.p = p;
   if (sizeof (p) == sizeof (unsigned short))
     id = u.us;
@@ -303,7 +299,7 @@ static int
 block_self (SCM queue, scm_i_pthread_mutex_t *mutex,
 	    const scm_t_timespec *waittime)
 {
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
   SCM q_handle;
   int err;
 
@@ -358,13 +354,13 @@ scm_i_pthread_key_t scm_i_thread_key;
    itself in TLS (rather than a pointer to some malloc'd memory) is not
    possible since thread objects may live longer than the actual thread they
    represent.  */
-SCM_THREAD_LOCAL scm_i_thread *scm_i_current_thread = NULL;
+SCM_THREAD_LOCAL scm_thread *scm_i_current_thread = NULL;
 
 #endif /* SCM_HAVE_THREAD_STORAGE_CLASS */
 
 
 static scm_i_pthread_mutex_t thread_admin_mutex = SCM_I_PTHREAD_MUTEX_INITIALIZER;
-static scm_i_thread *all_threads = NULL;
+static scm_thread *all_threads = NULL;
 static int thread_count;
 
 static SCM default_dynamic_state;
@@ -372,35 +368,27 @@ static SCM default_dynamic_state;
 /* Perform first stage of thread initialisation, in non-guile mode.
  */
 static void
-guilify_self_1 (struct GC_stack_base *base)
+guilify_self_1 (struct GC_stack_base *base, int needs_unregister)
 {
-  scm_i_thread t;
+  scm_thread t;
 
   /* We must arrange for SCM_I_CURRENT_THREAD to point to a valid value
      before allocating anything in this thread, because allocation could
      cause GC to run, and GC could cause finalizers, which could invoke
      Scheme functions, which need the current thread to be set.  */
 
+  memset (&t, 0, sizeof (t));
+
   t.pthread = scm_i_pthread_self ();
   t.handle = SCM_BOOL_F;
   t.result = SCM_BOOL_F;
-  t.freelists = NULL;
-  t.pointerless_freelists = NULL;
-  t.dynamic_state = NULL;
-  t.dynstack.base = NULL;
-  t.dynstack.top = NULL;
-  t.dynstack.limit = NULL;
   t.pending_asyncs = SCM_EOL;
   t.block_asyncs = 1;
   t.base = base->mem_base;
-#ifdef __ia64__
-  t.register_backing_store_base = base->reg_base;
-#endif
   t.continuation_root = SCM_EOL;
   t.continuation_base = t.base;
   scm_i_pthread_cond_init (&t.sleep_cond, NULL);
-  t.wake = NULL;
-  t.vp = NULL;
+  scm_i_vm_prepare_stack (&t.vm);
 
   if (pipe2 (t.sleep_pipe, O_CLOEXEC) != 0)
     /* FIXME: Error conditions during the initialization phase are handled
@@ -410,10 +398,11 @@ guilify_self_1 (struct GC_stack_base *base)
 
   t.exited = 0;
   t.guile_mode = 0;
+  t.needs_unregister = needs_unregister;
 
   /* The switcheroo.  */
   {
-    scm_i_thread *t_ptr = &t;
+    scm_thread *t_ptr = &t;
     
     GC_disable ();
     t_ptr = GC_generic_malloc (sizeof (*t_ptr), thread_gc_kind);
@@ -441,7 +430,7 @@ guilify_self_1 (struct GC_stack_base *base)
 static void
 guilify_self_2 (SCM dynamic_state)
 {
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
 
   t->guile_mode = 1;
 
@@ -450,13 +439,8 @@ guilify_self_2 (SCM dynamic_state)
   t->continuation_root = scm_cons (t->handle, SCM_EOL);
   t->continuation_base = t->base;
 
-  {
-    size_t size = SCM_INLINE_GC_FREELIST_COUNT * sizeof (void *);
-    t->freelists = scm_gc_malloc (size, "freelists");
-    t->pointerless_freelists = scm_gc_malloc (size, "atomic freelists");
-  }
-
   t->dynamic_state = scm_gc_typed_calloc (scm_t_dynamic_state);
+  t->dynamic_state->thread_local_values = scm_c_make_hash_table (0);
   scm_set_current_dynamic_state (dynamic_state);
 
   t->dynstack.base = scm_gc_malloc (16 * sizeof (scm_t_bits), "dynstack");
@@ -478,7 +462,7 @@ on_thread_exit (void *v)
   /* This handler is executed in non-guile mode.  Note that although
      libgc isn't guaranteed to see thread-locals, for this thread-local
      that isn't an issue as we have the all_threads list.  */
-  scm_i_thread *t = (scm_i_thread *) v, **tp;
+  scm_thread *t = (scm_thread *) v, **tp;
 
   t->exited = 1;
 
@@ -499,6 +483,10 @@ on_thread_exit (void *v)
       }
   thread_count--;
 
+  /* Prevent any concurrent or future marker from visiting this
+     thread.  */
+  t->handle = SCM_PACK (0);
+
   /* If there's only one other thread, it could be the signal delivery
      thread, so we need to notify it to shut down by closing its read pipe.
      If it's not the signal delivery thread, then closing the read pipe isn't
@@ -510,21 +498,25 @@ on_thread_exit (void *v)
 
   /* Although this thread has exited, the thread object might still be
      alive.  Release unused memory.  */
-  t->freelists = NULL;
-  t->pointerless_freelists = NULL;
+  for (size_t n = 0; n < SCM_INLINE_GC_FREELIST_COUNT; n++)
+    t->freelists[n] = t->pointerless_freelists[n] = NULL;
   t->dynamic_state = NULL;
   t->dynstack.base = NULL;
   t->dynstack.top = NULL;
   t->dynstack.limit = NULL;
-  {
-    struct scm_vm *vp = t->vp;
-    t->vp = NULL;
-    if (vp)
-      scm_i_vm_free_stack (vp);
-  }
+  scm_i_vm_free_stack (&t->vm);
+#if ENABLE_JIT
+  scm_jit_state_free (t->jit_state);
+#endif
+  t->jit_state = NULL;
+
+#ifdef SCM_HAVE_THREAD_STORAGE_CLASS
+  scm_i_current_thread = NULL;
+#endif
 
 #if SCM_USE_PTHREAD_THREADS
-  GC_unregister_my_thread ();
+  if (t->needs_unregister)
+    GC_unregister_my_thread ();
 #endif
 }
 
@@ -586,6 +578,8 @@ scm_i_init_thread_for_guile (struct GC_stack_base *base,
 	}
       else
 	{
+          int needs_unregister = 0;
+
 	  /* Guile is already initialized, but this thread enters it for
 	     the first time.  Only initialize this thread.
 	  */
@@ -593,10 +587,11 @@ scm_i_init_thread_for_guile (struct GC_stack_base *base,
 
           /* Register this thread with libgc.  */
 #if SCM_USE_PTHREAD_THREADS
-          GC_register_my_thread (base);
+          if (GC_register_my_thread (base) == GC_SUCCESS)
+            needs_unregister = 1;
 #endif
 
-	  guilify_self_1 (base);
+	  guilify_self_1 (base, needs_unregister);
 	  guilify_self_2 (dynamic_state);
 	}
       return 1;
@@ -637,7 +632,7 @@ with_guile (struct GC_stack_base *base, void *data)
 {
   void *res;
   int new_thread;
-  scm_i_thread *t;
+  scm_thread *t;
   struct with_guile_args *args = data;
 
   new_thread = scm_i_init_thread_for_guile (base, args->dynamic_state);
@@ -703,7 +698,7 @@ void *
 scm_without_guile (void *(*func)(void *), void *data)
 {
   void *result;
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
 
   if (t->guile_mode)
     {
@@ -776,7 +771,7 @@ unprotect_launch_data (launch_data *data)
 static void *
 really_launch (void *d)
 {
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
   unprotect_launch_data (d);
   /* The thread starts with asyncs blocked.  */
   t->block_asyncs++;
@@ -996,7 +991,7 @@ scm_lock_mutex (SCM mx)
 
 static inline SCM
 lock_mutex (enum scm_mutex_kind kind, struct scm_mutex *m,
-            scm_i_thread *current_thread, scm_t_timespec *waittime)
+            scm_thread *current_thread, scm_t_timespec *waittime)
 #define FUNC_NAME "lock-mutex"
 {
   scm_i_scm_pthread_mutex_lock (&m->lock);
@@ -1067,7 +1062,7 @@ SCM_DEFINE (scm_timed_lock_mutex, "lock-mutex", 1, 1, 0,
 {
   scm_t_timespec cwaittime, *waittime = NULL;
   struct scm_mutex *m;
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
   SCM ret;
 
   SCM_VALIDATE_MUTEX (1, mutex);
@@ -1133,7 +1128,7 @@ scm_try_mutex (SCM mutex)
    against the mutex kind.  */
 static inline void
 unlock_mutex (enum scm_mutex_kind kind, struct scm_mutex *m,
-              scm_i_thread *current_thread)
+              scm_thread *current_thread)
 #define FUNC_NAME "unlock-mutex"
 {
   scm_i_scm_pthread_mutex_lock (&m->lock);
@@ -1174,7 +1169,7 @@ SCM_DEFINE (scm_unlock_mutex, "unlock-mutex", 1, 0, 0, (SCM mutex),
 #define FUNC_NAME s_scm_unlock_mutex
 {
   struct scm_mutex *m;
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
 
   SCM_VALIDATE_MUTEX (1, mutex);
 
@@ -1297,7 +1292,7 @@ SCM_DEFINE (scm_make_condition_variable, "make-condition-variable", 0, 0, 0,
 
 static inline SCM
 timed_wait (enum scm_mutex_kind kind, struct scm_mutex *m, struct scm_cond *c,
-            scm_i_thread *current_thread, scm_t_timespec *waittime)
+            scm_thread *current_thread, scm_t_timespec *waittime)
 #define FUNC_NAME "wait-condition-variable"
 {
   scm_i_scm_pthread_mutex_lock (&m->lock);
@@ -1406,7 +1401,7 @@ SCM_DEFINE (scm_timed_wait_condition_variable, "wait-condition-variable", 2, 1,
   scm_t_timespec waittime_val, *waittime = NULL;
   struct scm_cond *c;
   struct scm_mutex *m;
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
   SCM ret;
 
   SCM_VALIDATE_CONDVAR (1, cond);
@@ -1521,7 +1516,7 @@ scm_std_select (int nfds,
 {
   fd_set my_readfds;
   int res, eno, wakeup_fd;
-  scm_i_thread *t = SCM_I_CURRENT_THREAD;
+  scm_thread *t = SCM_I_CURRENT_THREAD;
   struct select_args args;
 
   if (readfds == NULL)
@@ -1530,40 +1525,45 @@ scm_std_select (int nfds,
       readfds = &my_readfds;
     }
 
-  while (scm_i_prepare_to_wait_on_fd (t, t->sleep_pipe[1]))
-    SCM_TICK;
-
-  wakeup_fd = t->sleep_pipe[0];
-  FD_SET (wakeup_fd, readfds);
-  if (wakeup_fd >= nfds)
-    nfds = wakeup_fd+1;
-
-  args.nfds = nfds;
-  args.read_fds = readfds;
-  args.write_fds = writefds;
-  args.except_fds = exceptfds;
-  args.timeout = timeout;
-
-  /* Explicitly cooperate with the GC.  */
-  scm_without_guile (do_std_select, &args);
-
-  res = args.result;
-  eno = args.errno_value;
-
-  scm_i_wait_finished (t);
-
-  if (res > 0 && FD_ISSET (wakeup_fd, readfds))
+  if (scm_i_prepare_to_wait_on_fd (t, t->sleep_pipe[1]))
     {
-      char dummy;
-      full_read (wakeup_fd, &dummy, 1);
+      eno = EINTR;
+      res = -1;
+    }
+  else
+    {
+      wakeup_fd = t->sleep_pipe[0];
+      FD_SET (wakeup_fd, readfds);
+      if (wakeup_fd >= nfds)
+        nfds = wakeup_fd+1;
 
-      FD_CLR (wakeup_fd, readfds);
-      res -= 1;
-      if (res == 0)
-	{
-	  eno = EINTR;
-	  res = -1;
-	}
+      args.nfds = nfds;
+      args.read_fds = readfds;
+      args.write_fds = writefds;
+      args.except_fds = exceptfds;
+      args.timeout = timeout;
+
+      /* Explicitly cooperate with the GC.  */
+      scm_without_guile (do_std_select, &args);
+
+      res = args.result;
+      eno = args.errno_value;
+
+      scm_i_wait_finished (t);
+
+      if (res > 0 && FD_ISSET (wakeup_fd, readfds))
+        {
+          char dummy;
+          full_read (wakeup_fd, &dummy, 1);
+
+          FD_CLR (wakeup_fd, readfds);
+          res -= 1;
+          if (res == 0)
+            {
+              eno = EINTR;
+              res = -1;
+            }
+        }
     }
   errno = eno;
   return res;
@@ -1680,7 +1680,7 @@ SCM_DEFINE (scm_all_threads, "all-threads", 0, 0, 0,
      of the way GC is done.
   */
   int n = thread_count;
-  scm_i_thread *t;
+  scm_thread *t;
   SCM list = scm_c_make_list (n, SCM_UNSPECIFIED), *l;
 
   scm_i_pthread_mutex_lock (&thread_admin_mutex);
@@ -1713,7 +1713,7 @@ int
 scm_c_thread_exited_p (SCM thread)
 #define FUNC_NAME  s_scm_thread_exited_p
 {
-  scm_i_thread *t;
+  scm_thread *t;
   SCM_VALIDATE_THREAD (1, thread);
   t = SCM_I_THREAD_DATA (thread);
   return t->exited;
@@ -1782,7 +1782,7 @@ scm_threads_prehistory (void *base)
 		 GC_MAKE_PROC (GC_new_proc (thread_mark), 0),
 		 0, 1);
 
-  guilify_self_1 ((struct GC_stack_base *) base);
+  guilify_self_1 ((struct GC_stack_base *) base, 0);
 }
 
 scm_t_bits scm_tc16_thread;
@@ -1792,7 +1792,7 @@ scm_t_bits scm_tc16_condvar;
 static void
 scm_init_ice_9_threads (void *unused)
 {
-#include "libguile/threads.x"
+#include "threads.x"
 
   cancel_thread_var =
     scm_module_variable (scm_current_module (),
@@ -1808,7 +1808,7 @@ scm_init_ice_9_threads (void *unused)
 void
 scm_init_threads ()
 {
-  scm_tc16_thread = scm_make_smob_type ("thread", sizeof (scm_i_thread));
+  scm_tc16_thread = scm_make_smob_type ("thread", sizeof (scm_thread));
   scm_set_smob_print (scm_tc16_thread, thread_print);
 
   scm_tc16_mutex = scm_make_smob_type ("mutex", sizeof (struct scm_mutex));
@@ -1834,65 +1834,4 @@ scm_init_threads_default_dynamic_state ()
 }
 
 
-/* IA64-specific things.  */
 
-#ifdef __ia64__
-# ifdef __hpux
-#  include <sys/param.h>
-#  include <sys/pstat.h>
-void *
-scm_ia64_register_backing_store_base (void)
-{
-  struct pst_vm_status vm_status;
-  int i = 0;
-  while (pstat_getprocvm (&vm_status, sizeof (vm_status), 0, i++) == 1)
-    if (vm_status.pst_type == PS_RSESTACK)
-      return (void *) vm_status.pst_vaddr;
-  abort ();
-}
-void *
-scm_ia64_ar_bsp (const void *ctx)
-{
-  uint64_t bsp;
-  __uc_get_ar_bsp (ctx, &bsp);
-  return (void *) bsp;
-}
-# endif /* hpux */
-# ifdef linux
-#  include <ucontext.h>
-void *
-scm_ia64_register_backing_store_base (void)
-{
-  extern void *__libc_ia64_register_backing_store_base;
-  return __libc_ia64_register_backing_store_base;
-}
-void *
-scm_ia64_ar_bsp (const void *opaque)
-{
-  const ucontext_t *ctx = opaque;
-  return (void *) ctx->uc_mcontext.sc_ar_bsp;
-}
-# endif /* linux */
-# ifdef __FreeBSD__
-#  include <ucontext.h>
-void *
-scm_ia64_register_backing_store_base (void)
-{
-  return (void *)0x8000000000000000;
-}
-void *
-scm_ia64_ar_bsp (const void *opaque)
-{
-  const ucontext_t *ctx = opaque;
-  return (void *)(ctx->uc_mcontext.mc_special.bspstore
-                  + ctx->uc_mcontext.mc_special.ndirty);
-}
-# endif /* __FreeBSD__ */
-#endif /* __ia64__ */
-
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
