@@ -1,6 +1,6 @@
 ;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2019 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -143,7 +143,7 @@ by a label, respectively."
                 (intmap-add! uses label u)))
       (match cont
         (($ $kfun src meta self)
-         (return (intset self) empty-intset))
+         (return (if self (intset self) empty-intset) empty-intset))
         (($ $kargs _ _ ($ $continue k src exp))
          (match exp
            ((or ($ $const) ($ $const-fun) ($ $code))
@@ -151,7 +151,8 @@ by a label, respectively."
            (($ $call proc args)
             (return (get-defs k) (intset-add (vars->intset args) proc)))
            (($ $callk _ proc args)
-            (return (get-defs k) (intset-add (vars->intset args) proc)))
+            (let ((args (vars->intset args)))
+              (return (get-defs k) (if proc (intset-add args proc) args))))
            (($ $primcall name param args)
             (return (get-defs k) (vars->intset args)))
            (($ $values args)
@@ -324,7 +325,7 @@ the definitions that are live before and after LABEL, as intsets."
         (($ $kclause arity body alternate)
          (get-defs label))
         (($ $kfun src meta self)
-         (intset self))
+         (if self (intset self) empty-intset))
         (($ $ktail)
          empty-intset))))
    cps
@@ -350,8 +351,9 @@ is an active call."
                      (intset-subtract (intset-add (list->intset args) proc)
                                       (intmap-ref live-out label)))
                     (($ $kargs _ _ ($ $continue _ _ ($ $callk _ proc args)))
-                     (intset-subtract (intset-add (list->intset args) proc)
-                                      (intmap-ref live-out label)))
+                     (let ((args (list->intset args)))
+                       (intset-subtract (if proc (intset-add args proc) args)
+                                        (intmap-ref live-out label))))
                     (($ $kargs _ _ ($ $continue k _($ $values args)))
                      (match (intmap-ref cps k)
                        (($ $ktail) (list->intset args))
@@ -587,7 +589,7 @@ are comparable with eqv?.  A tmp slot may be used."
          (($ $call proc args)
           (add-call-shuffles label k (cons proc args) shuffles))
          (($ $callk _ proc args)
-          (add-call-shuffles label k (cons proc args) shuffles))
+          (add-call-shuffles label k (if proc (cons proc args) args) shuffles))
          (($ $values args)
           (add-values-shuffles label k args shuffles))
          (_ shuffles)))
@@ -629,7 +631,8 @@ are comparable with eqv?.  A tmp slot may be used."
            (($ $continue _ _ ($ $call proc args))
             (call-size label (1+ (length args)) size))
            (($ $continue _ _ ($ $callk _ proc args))
-            (call-size label (1+ (length args)) size))
+            (let ((nclosure (if proc 1 0)))
+              (call-size label (+ nclosure (length args)) size)))
            (($ $continue _ _ ($ $values args))
             (shuffle-size (get-shuffles label) size))
            (_ size))))
@@ -640,22 +643,27 @@ are comparable with eqv?.  A tmp slot may be used."
   (intmap-fold measure-cont cps minimum-frame-size))
 
 (define (allocate-args cps)
-  (intmap-fold (lambda (label cont slots)
-                 (match cont
-                   (($ $kfun src meta self)
-                    (intmap-add! slots self 0))
-                   (($ $kclause arity body alt)
-                    (match (intmap-ref cps body)
-                      (($ $kargs names vars)
-                       (let lp ((vars vars) (slots slots) (n 1))
-                         (match vars
-                           (() slots)
-                           ((var . vars)
-                            (lp vars
-                                (intmap-add! slots var n)
-                                (1+ n))))))))
-                   (_ slots)))
-               cps empty-intmap))
+  (match (intmap-ref cps (intmap-next cps))
+    (($ $kfun _ _ has-self?)
+     (intmap-fold (lambda (label cont slots)
+                    (match cont
+                      (($ $kfun src meta self)
+                       (if has-self?
+                           (intmap-add! slots self 0)
+                           slots))
+                      (($ $kclause arity body alt)
+                       (match (intmap-ref cps body)
+                         (($ $kargs names vars)
+                          (let lp ((vars vars) (slots slots)
+                                   (n (if has-self? 1 0)))
+                            (match vars
+                              (() slots)
+                              ((var . vars)
+                               (lp vars
+                                   (intmap-add! slots var n)
+                                   (1+ n))))))))
+                      (_ slots)))
+                  cps empty-intmap))))
 
 (define-inlinable (add-live-slot slot live-slots)
   (logior live-slots (ash 1 slot)))
@@ -719,7 +727,7 @@ are comparable with eqv?.  A tmp slot may be used."
          (($ $call proc args)
           (allocate-call label (cons proc args) slots))
          (($ $callk _ proc args)
-          (allocate-call label (cons proc args) slots))
+          (allocate-call label (if proc (cons proc args) args) slots))
          (($ $values args)
           (allocate-values label k args slots))
          (_ slots)))
@@ -784,7 +792,9 @@ are comparable with eqv?.  A tmp slot may be used."
        (($ $kargs _ _ (or ($ $branch) ($ $prompt) ($ $throw)))
         representations)
        (($ $kfun src meta self)
-        (intmap-add representations self 'scm))
+        (if self
+            (intmap-add representations self 'scm)
+            representations))
        (($ $kclause arity body alt)
         (fold1 (lambda (var representations)
                  (intmap-add representations var 'scm))
@@ -980,7 +990,8 @@ are comparable with eqv?.  A tmp slot may be used."
              (($ $continue k src ($ $call proc args))
               (allocate-call label k (cons proc args) slots call-allocs live))
              (($ $continue k src ($ $callk _ proc args))
-              (allocate-call label k (cons proc args) slots call-allocs live))
+              (allocate-call label k (if proc (cons proc args) args)
+                             slots call-allocs live))
              (($ $continue k src ($ $values args))
               (allocate-values label k args slots call-allocs))
              (($ $prompt k kh src escape? tag)
